##################################################################################
##################################################################################
# SEM-5_EXERCISE-1: BAYESIAN MODELING IN STAN AND R
# TOPICS: REGRESSION AND MULTILEVEL MODELING
#
# THIS SYNTAX INCLUDES ALL ANALYSES FOR THE EXERCISES
# EXTERNAL FILES:
# regression.stan: SOLUTION (DEMO)
# mlm1_start.stan: EXERCISE 2 and 3 (save as mlm2_start.stan and extend the file)
# SOLUTIONS FOR THESE MODEL ARE PROVIDED LATER
# mlm1_solution.stan 
# mlm2_solution.stan
##################################################################################
##################################################################################

#setwd("c:/holger/workshops/nonlinear/exercises/day3")
#dat1 <- read.table("pisa_manifest.dat", header = TRUE)

# packages
library(lme4)  # mlm ML estimator
library(rstan) # run stan via r
# always include these two options for stan (recommendation from package builders)
rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

###############
dat1 <- read.table("../data/math_kft.dat", header = TRUE)

# sample size
N <- dim(dat1)[1]
head(dat1)

# initial graphs
par(mfrow=c(1,2))
scatter.smooth(dat1$on,dat1$read,xlab="online",ylab="read")
scatter.smooth(dat1$att,dat1$read,xlab="att",ylab="read")
# => shows quadratic(?) relationship btw on and read
# => shows linear relationship btw att and read

par(mfrow=c(1,1))
boxplot(dat1$read~dat1$school)
# => read also depends on the schools (==cluster) the students go to 
# we take that later into account

#########################################
#########################################
# EXERCISE 1: REGRESSION (DEMO)
#########################################
#########################################

#########################################
# normal regression (OLS)
#########################################
reg1 <- lm(read~on+att+on:att+I(on^2)+I(att^2),data=dat1)
summary(reg1)
# => significant quadratic effect of on

# residual analysis
plot(reg1)
# => shows normal and homoskedastic residual

#########################################
# Bayes regression
# For stan, one needs 3 steps to run a model:
# 1. Read in an (external) model file
# 2. Compile the model (i.e. translate the model in a c++ code)
# 3. Run the model (Hamiltonian MC)
#########################################
# read in the model
rt1  <- stanc("regression.stan")
# convert the model (c++ compiler)
sm1  <- stan_model(stanc_ret = rt1, verbose=FALSE)

# data file as a list of variables
# these variables (x,y...) need to be declared in stan
# instead of variables, we will use matrices later
datstan <- list(N,y=dat1$read,x1=dat1$on,x2=dat1$att)
# run the model
fit1 <- sampling(sm1, data=datstan)

#########################################
# After that one needs to check the convergence
# 1. Rhat (general index describing convergence); should be < 1.1
# 2. Trace plots (for individiual parameters): Allows to identify problematic parameters
# 3. Density plots (distribution): Further investigation of distribution 
#    (e.g., check if mean is reasonable statistic)
#########################################
# Rhat
stan_rhat(fit1)

# trace plots
stan_trace(fit1,"b0")
stan_trace(fit1,"b1")
stan_trace(fit1,"sigmay")

# density plots
stan_dens(fit1,"b0",separate_chains =T)
stan_dens(fit1,"b1",separate_chains =T)
stan_dens(fit1,"sigmay",separate_chains =T)

#########################################
# Results
#########################################
params <- c("b0","b1","sigmay")
print(fit1,pars=params)


#########################################
#########################################
# EXERCISE 2: multilevel model with random intercept
#########################################
#########################################
# the syntax in lmer extends the lm-notation by including a fixed and a random part:
# lmer(y ~ fixed + (random|clustering), data=dat1)
# Accordingly, a random intercept for the classes is added by "(1|class)"
reg2 <- lmer(read~on+att+on:att+I(on^2)+I(att^2)+(1|school),data=dat1)
summary(reg2)

# Supershort version of residual analysis:
plot(reg2)
# => The residuals are homoskedastic (and probably also normal)

# Does the clustering affect the reading skills?
# 1. This is a comparison model without random intercept
reg2a <- glm(read~on+att+on:att+I(on^2)+I(att^2),data=dat1)
# 2. Model comparison via "anova" provides a likelihood-ratio (aka deviance) test
anova(reg2,reg2a)
# note: Df are actually the number of parameters (BUG)

#########################################
# Bayesian version of random intercept model
#########################################

########################
# Data preparation
########################
Ns   <- length(levels(as.factor(dat1$school)))

# new list of data
datstan2 <- list(N,y=dat1$read,x1=dat1$on,x2=dat1$att,
                 Ns=Ns,school=dat1$school)

# stan
rt2  <- stanc("mlm1_start.stan")
#rt2  <- stanc("mlm1_solution.stan")
sm2  <- stan_model(stanc_ret = rt2, verbose=FALSE)
fit2 <- sampling(sm2, data=datstan2) #12 sec

# check the convergence
stan_rhat(fit2)
params <- c("b0","b1","sigmay","sigmau0")
stan_trace(fit2,params)
stan_dens(fit2,"b1",separate_chains=T)
stan_dens(fit2,"sigmau0",separate_chains=T)

print(fit2,pars=params)

# this provides the factor scores
output  <- as.data.frame(fit2)
u0hat <- apply(output[paste0("u0[",1:Ns,"]")],2,mean)
mean(u0hat)
sd(u0hat)

plot(ranef(reg2),u0hat)

#########################################
#########################################
# EXERCISE 3: mlm with random intercept & slope (u0/u1)
#########################################
#########################################

# the ML version is provided here:
reg3 <- lmer(read~on+att+on:att+I(on^2)+I(att^2)+
               (1|school)+(0+on|school)+(0+att|school),data=dat1)
summary(reg3)

random.reg3 <- as.data.frame(VarCorr(reg3))
# SD's
sd.reg3 <- round(random.reg3[,5],2)
names(sd.reg3) <- c("u0","u1","u2","eps")

# stan
rt3  <- stanc("mlm2_start.stan")
#rt3  <- stanc("mlm2_solution.stan")
sm3  <- stan_model(stanc_ret = rt3, verbose=FALSE)
fit3 <- sampling(sm3, data=datstan2)

# check the convergence
stan_rhat(fit3)
stan_trace(fit3,"b1")
stan_dens(fit3,"b1",separate_chains =T)
# random terms
stan_trace(fit3,"sigmau0")
stan_trace(fit3,"sigmau1")
stan_trace(fit3,"sigmau2")

stan_dens(fit3,"sigmau0",separate_chains =T)
stan_dens(fit3,"sigmau1",separate_chains =T)
stan_dens(fit3,"sigmau2",separate_chains =T)

params <- c("b0","b1","sigmay","sigmau0","sigmau1","sigmau2")
print(fit3,pars=params)
# => This model does not converge. The random slopes cause problems























