##################################################################################
##################################################################################
# SEM-5_EXERCISE-2: BAYESIAN MODELING IN STAN AND R
# TOPICS: SEM and Multilevel SEM (with interaction effects)
#
# THIS SYNTAX INCLUDES ALL ANALYSES FOR THE EXERCISES
# EXTERNAL FILES:
# sem1
# sem1b: Cholesky version EXERCSISE 1
# mlsem1: EXERCSISE 2
# SOLUTIONS:
# sem1a: EXERCSISE 1
# mlsem2: EXERCSISE 2
##################################################################################
##################################################################################

#setwd("c:/holger/workshops/nonlinear/exercises/day1/01_pi_approach")
#dat.kj <- read.table("data_kj.dat", header = TRUE)
#setwd("c:/holger/workshops/nonlinear/exercises/day3")

# packages
library(lavaan)
library(rstan)
rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

#########################################
#########################################
# EXERCISE 1: simple sem with Kenny-Judd data (DEMO)
#########################################
#########################################

dat.kj <- read.table("../data/data_kj.dat",header=T)

# center data
dat.kj$v1 <- dat.kj$v1-mean(dat.kj$v1)
dat.kj$v2 <- dat.kj$v2-mean(dat.kj$v2)
dat.kj$s1 <- dat.kj$s1-mean(dat.kj$s1)
dat.kj$s2 <- dat.kj$s2-mean(dat.kj$s2)

N <- dim(dat.kj)[1]

#########################################
# simple linear sem with KJ data (as in SEM-3)
#########################################
model.lin <- '
xi1  =~ v1+1*v2
xi2  =~ s1+1*s2
eta  =~ c1+1*c2
eta ~ 1+ xi1 + xi2
v1 ~ 0*1
v2 ~ 0*1
s1 ~ 0*1
s2 ~ 0*1
c1 ~ 0*1
c2 ~ 0*1
'

sem.lin <- sem(model.lin,dat=dat.kj)
summary(sem.lin)

#########################################
# Bayes sem with interaction effect (DEMO and EXERCISE)
#########################################
# data: for simplicity we use matrices 
# y (N x 2) and x (N x 4) for the indicator variables:
# Note: We do not need product indicators!
datstan <- list(N,y=cbind(dat.kj$c1,dat.kj$c2),
                x=cbind(dat.kj$v1,dat.kj$v2,dat.kj$s1,dat.kj$s2))

# stan
rt1  <- stanc("sem1.stan")   # takes 322sec
#rt1  <- stanc("sem1a.stan") # model including quadratic effects
#rt1  <- stanc("sem1b.stan") # cholesky version; takes 100s
sm1  <- stan_model(stanc_ret = rt1, verbose=FALSE)
fit1 <- sampling(sm1, data=datstan)

# results
stan_rhat(fit1)
stan_trace(fit1,"b1")
stan_dens(fit1,"b1",separate_chains =T)

params <- c("b0","b1","sigmay","sigmax","sigmaeta","phi")
print(fit1,pars=params)


#########################################
#########################################
# EXERCISE 2. Bayes ML-SEM (DEMO)
#########################################
#########################################
dat1 <- read.table("../data/pisa_mlsem.dat",header=T)
#dat1 <- read.table("pisa_mlsem.dat",header=T)
head(dat1)

# NOTE: att is now a school level variable
# that indicates the average reading attitude in a school

########################
# Data preparation
########################
N  <- dim(dat1)[1]
Ns <- length(levels(as.factor(dat1$school)))

# recode the level 2 variable (from format N to Ns)
# We need that for the final model mlsem3.stan
x2 <- matrix(0,Ns,3)
school0 <- as.factor(dat1$school)
for(i in 1:Ns){#i<-1
  x2[,] <- apply(dat1[levels(school0)[i]==school0,paste0("att",1:3)],2,mean)
}

y  <- as.matrix(dat1[,paste0("read",1:3)])
x1 <- as.matrix(dat1[,paste0("on",1:3)])
x2 <- as.matrix(dat1[,paste0("att",1:3)])

########################
# Random intercept model: without x2
########################
data1 <- list(y=y,x1=x1,N=N,Ns=Ns,school=dat1$school)

# stan
rt1  <- stanc("mlsem1.stan") #takes 70sec
sm1  <- stan_model(stanc_ret = rt1, verbose=FALSE)
fit1 <- sampling(sm1, data=data1)

# results
stan_rhat(fit1)
stan_trace(fit1,"b1")
stan_dens(fit1,"b1",separate_chains =T)

params <- c("b0","b1","ly","lx1","sigmay","sigmax1","sigmaeta","sigmau0","sigmaxi")
print(fit1,pars=params)

# this provides the factor scores
output  <- as.data.frame(fit1)
u0hat <- apply(output[paste0("u0[",1:Ns,"]")],2,mean)
mean(u0hat)
sd(u0hat)

hist(u0hat)

########################
# Random intercept model with level 2 variable
# and cross level interaction
########################
data2 <- list(y=y,x1=x1,x2=x2,N=N,Ns=Ns,school=dat1$school)

cor(y,x1)

# stan
rt2  <- stanc("mlsem2.stan")
sm2  <- stan_model(stanc_ret = rt2, verbose=FALSE)
fit2 <- sampling(sm2, data=data2)

# results
stan_rhat(fit2)
stan_trace(fit2,"lx")
stan_trace(fit2,"b1")
stan_dens(fit2,"b1",separate_chains =T)
stan_dens(fit2,"sigmax2",separate_chains =T)

params <- c("b0","b1","ly","lx",
            "sigmay","sigmax",
            "sigmaeta","sigmau0","phixi")
print(fit2,pars=params)






