data {
  int<lower=0> N;          // number of students (level1)
  int<lower=0> Ns;         // number of schools (level2)
  matrix[N,3]  y;          // reading skills
  matrix[N,3]  x1;         // online
  matrix[N,3]  x2;         // attitude
  int<lower=0> school[N];  // schoolid
}

parameters {
  real         b0;         // intercept structural model
  vector[5]    b1;         // fixed effects structural model
  vector<lower=0>[2]    ly;         // factor loadings y
  vector<lower=0>[4]    lx;         // factor loadings x1
  
  vector<lower=0>[3] sigmay;   // residual SD L1 for y
  vector<lower=0>[6] sigmax;  // residual SD L1 for x1
  vector<lower=0>[2] sigmaxi;  // SD L1 xi1
  real<lower=0>      sigmaeta; // residual SD L1 eta
  real<lower=0>      sigmau0;  // residual SD L2

  cholesky_factor_corr[2] L1;
  
  vector[N]   eta;   // latent dv L1
  matrix[N,2] zi;    // latent predictor L1 CHOLESKY
  vector[Ns]  u0;    // random effects
}

transformed parameters {
  matrix[N,3]  muy;   // E[y|eta]
  matrix[N,3]  mux1;  // E[x1|xi1]
  matrix[N,3]  mux2;  // E[x2|xi2]
  vector[N]    mueta; // E[eta|xi1,xi2]
  matrix[N,2]  xi;    // 

  xi = zi*diag_pre_multiply(sigmaxi,L1)'; // this generates xi
  
  for (i in 1:N){
    ///////////////////////////////////////////////
    // measurement models L1
    ///////////////////////////////////////////////
    muy[i,1] =       eta[i];
    muy[i,2] = ly[1]*eta[i];
    muy[i,3] = ly[2]*eta[i];
  
    mux1[i,1] =       xi[i,1];
    mux1[i,2] = lx[1]*xi[i,1];
    mux1[i,3] = lx[2]*xi[i,1];
    
    mux2[i,1] =       xi[i,2];
    mux2[i,2] = lx[3]*xi[i,2];
    mux2[i,3] = lx[4]*xi[i,2];
    
    ///////////////////////////////////////////////
    // structural model
    ///////////////////////////////////////////////
    mueta[i] = b0 + 
               b1[1]*xi[i,2] +         //linear effect of on
               b1[2]*xi[i,2] +         //linear effect of att
               b1[3]*xi[i,1]*xi[i,1] + //quadratic effect of on
               b1[4]*xi[i,1]*xi[i,2] + //interaction
               b1[5]*xi[i,2]*xi[i,2] + //quadratic effect of att
               u0[school[i]];
  
  }
  
}

model {
  ///////////////////////////////////
  // distribution of y,x
  ///////////////////////////////////
  for(j in 1:3){
    y[,j]  ~ normal(muy[,j],sigmay[j]);
    x1[,j] ~ normal(mux1[,j],sigmax[j]);
    x2[,j] ~ normal(mux2[,j],sigmax[j+3]);
  }

  ///////////////////////////////////
  // distribution of latent variables
  ///////////////////////////////////
  eta ~ normal(mueta,sigmaeta);
  to_vector(zi) ~ normal(0,1);
  
  ///////////////////////////////////
  // distribution of random effects
  ///////////////////////////////////
  u0 ~ normal(0,sigmau0);
  
  ///////////////////////////////////
  // priors measurement model
  ///////////////////////////////////
  ly ~ normal(0,1);
  lx ~ normal(0,1);
  
  sigmay ~ cauchy(0,2.5);
  sigmax ~ cauchy(0,2.5);
  
  ///////////////////////////////////
  // priors structural model
  ///////////////////////////////////
  // fixed effects
  b0 ~ normal(0,1);
  b1 ~ normal(0,1);
  
  // variances and covariances
  sigmaeta ~ cauchy(0,2.5);
  sigmau0  ~ cauchy(0,2.5);
  sigmaxi  ~ cauchy(0,2.5);
  
  // cholesky version (4/5)
  L1 ~ lkj_corr_cholesky(2);
  
}

generated quantities{
  // covariance matrix
  matrix[2,2] phixi;
  phixi = diag_pre_multiply(sigmaxi,L1)*diag_pre_multiply(sigmaxi,L1)';

}





